#ifndef __TAPTEMPOLEASTSQUARES_HPP__
#define __TAPTEMPOLEASTSQUARES_HPP__
/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* TapTempoLeastSquares.hpp
   Gabriel M. Beddingfield
   2008-02-10

 */

#include "TapTempoInterface.hpp"

// TapTempoLeastSqures
// Tn+1 determined by a least squares fit of T0..Tn
class TapTempoLeastSquares : public TapTempoInterface
{
public:
    TapTempoLeastSquares() :
	m_taps_received(0),
	m_last_tap(0),
	m_sum_taps(0),
	m_sum_i_taps(0)
	{ m_taps_required = 4; }

    usecs_t tap(usecs_t tap_, usecs_t& cycles_per_beat);
    virtual const char* name(void);
    virtual void invalidate(void);

private:
    void invalidate_(void) {
	m_sum_taps = m_sum_i_taps = 0.0;
	m_taps_received = 0;
    }
    
    usecs_t lastTap(usecs_t& cycles_per_beat);   
    
private:
    unsigned m_taps_received;
    double m_last_tap;  // In seconds (frame/m_frame_rate)
    double m_sum_taps;
    double m_sum_i_taps;
};
    
#endif // __TAPTEMPO_HPP__
