#ifndef __MIDILISTENER_HPP__
#define __MIDILISTENER_HPP__
/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Performances           *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* MidiListener.cpp
   Gabriel M. Beddingfield
   2008-02-08

 */

#ifdef USE_JACK_MIDI
#include <jack/types.h>
#else
#include <QtCore/QThread>
#include <alsa/asoundlib.h>
#include <alsa/seq.h>
#endif

namespace InConcert
{
    class TimeBase;

    class MidiListener
#ifndef USE_JACK_MIDI
        : public QThread
#endif
    {

    public:
        MidiListener(TimeBase* tb);
        ~MidiListener();
  
#ifdef USE_JACK_MIDI
        static int ProcessCallback(jack_nframes_t nframes, void *arg);
#endif
    protected:

#ifdef USE_JACK_MIDI
        int _ProcessCallback(jack_nframes_t nframes);
#else
        void run(void);
#endif

    private:
        int ProcessMidiEvent(const unsigned char* buf, const size_t len);

    private:
        unsigned char m_chan; // channel we're listening on, 0x00-0x0F
        TimeBase* m_timebase;
#ifdef USE_JACK_MIDI
        jack_client_t* m_client;
        jack_port_t* m_input;
#else
        snd_seq_t* m_seq;
        int m_input;
#endif
    };

}
    
#endif // __MIDILISTENER_HPP__
