/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* Mutex.cpp
   Gabriel M. Beddingfield
   2008-01-29

 */

#include <stdexcept>
#include <pthread.h>
#include "Mutex.hpp"
#include "MutexPrivate.hpp"

using namespace std;
using namespace InConcert;

Mutex::Mutex(void) throw(runtime_error)
{
    d = new MutexPrivate;
    if(pthread_mutex_init(&d->mutex, 0)) {
        throw runtime_error("Could not initialize mutex");
    }
}

Mutex::Mutex(const Mutex& /*o*/) throw(runtime_error)
{
    // Don't copy the mutex!!!
    d = new MutexPrivate;
    if(pthread_mutex_init(&d->mutex, 0)) {
        throw runtime_error("Could not initialize mutex");
    }
}

Mutex::~Mutex()
{
    unlock();
    pthread_mutex_destroy(&d->mutex);
}

void Mutex::lock(void) throw(runtime_error)
{
    if(pthread_mutex_lock(&d->mutex)) {
        throw runtime_error("Error locking mutex");
    }
    state = Locked;
}

bool Mutex::trylock(void) throw()
{
    if(0 == pthread_mutex_trylock(&d->mutex)) {
        state = Unlocked;
        return true;
    }
    return false;
}

bool Mutex::unlock(void) throw()
{
    if(0 == pthread_mutex_unlock(&d->mutex)) {
        state = Unlocked;
        return true;
    }
    return false;
}

