#ifndef __MUTEXLOCKER_HPP__
#define __MUTEXLOCKER_HPP__
/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* MutexLocker.cpp
   Gabriel M. Beddingfield
   2008-01-29

 */

#include "Mutex.hpp"

namespace InConcert
{
    class Mutex;

    // class MutexLocker
    //
    // Provides an exception-safe interfact to a Mutex.
    // If an exception is called, the destructor will
    // be called and the mutex will be unlocked.
    // Otherwise you have to do something like:
    //
    class MutexLocker
    {
        Mutex* mutex;
    public:
        MutexLocker(Mutex* m) : mutex(m) {
            mutex->lock();
        }
    
        ~MutexLocker() {
            mutex->unlock();
        }
    
        bool unlock(void) {
            return mutex->unlock();
        }
    };
    
} // namespace InConcert
    
#endif // __MUTEXLOCKER_HPP__
